# -*- coding: utf-8 -*-

TWITTER_ACCESS_TOKEN = "361149228-jwZ0jYLRxyIGc6QvASTVqH1MdPnZZ09WPRUEjYVR"
TWITTER_ACCESS_TOKEN_SECRET = "ZdygvVGtA69fw6nfDN1HREuRET4wI5P8cEA74Mj7pEc"
TWITTER_CONSUMER_KEY = '1mtBRl05RVxhI41KmmrzA'
TWITTER_CONSUMER_SECRET = 'CjkCMjHDmH1UQgnrCxPuFPDibSRBZcVONyVaBORos'

RDIO_API_KEY = "z6qp9bt9b8z7368yag4hgv5c"
RDIO_SHARED_SECRET = "VWYggKrv6z"
